
#pragma once

#include<windows.h>
#include<stdio.h>
#include"PointerList.h"

class NARReader
{
protected:
	HANDLE narHandle;
	void* pMapping;
	HANDLE hMapping;

	PointerList* fileList;
public:
	NARReader(char* archiveName);
	int GetFileCount();
	void* ReadFile(int index);
	char* ReadFileName(int index);
	void* ReadFile(char* fileName);
	DWORD ReadFileSize(char* fileName);
	DWORD ReadFileSize(int index);
	virtual ~NARReader();
};


